<?php

namespace Shavshukov\EventAppBase\FileReaders;

use React\Stream\WritableStreamInterface;
use React\Stream\ReadableResourceStream;
use React\EventLoop\LoopInterface;
use React\Promise;

class StreamedFileReader implements FileReaderInterface
{
    /**
     * Event loop instance
     *
     * @var \React\EventLoop\LoopInterface 
     */
    protected $loop;
    
    /**
     * Constructor
     * 
     * @param \React\EventLoop\LoopInterface $loop
     */
    public function __construct(LoopInterface $loop)
    {
        $this->loop = $loop;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getContents(string $fileName): Promise\PromiseInterface
    {
        return new Promise\Promise(function (\Closure $resolve) use ($fileName) {
            $buffer = '';
            $readStream = new ReadableResourceStream(fopen($fileName, 'r'), $this->loop);
            
            if ($readStream->isReadable()) {
                $readStream->on('data', function ($data) use (&$buffer) {
                    $buffer .= $data;
                });

                $readStream->on('end', function () use ($resolve, &$buffer) {
                    $resolve($buffer);
                });
            } else {
                $resolve($buffer);
            }
        });
    }
    
    /**
     * {@inheritDoc}
     */
    public function pipeContents(string $fileName, WritableStreamInterface $writeStream, array $options = [])
    {
        (new ReadableResourceStream(fopen($fileName, 'r'), $this->loop, $options))->pipe($writeStream);
    }
    
    /**
     * {@inheritDoc}
     */
    public function asResponseArgument(string $fileName, array $options = [])
    {
        return (new ReadableResourceStream(fopen($fileName, 'r'), $this->loop, $options));
    }
}
