<?php

namespace Shavshukov\EventAppBase\Interfaces;

interface ModelInterface extends StorageInterface, ValidatableInterface
{

    /**
     * Check that value at path exists
     * 
     * @param string $path
     * @return bool
     */
    public function has(string $path): bool;
        
    /**
     * Get value at path
     * 
     * @param string $path
     * @param mixed $default
     * @return mixed
     */
    public function get(string $path, $default = null);
    
    /**
     * Get underlying array of the model
     * 
     * @return array
     */
    public function all(): array;
    
    /**
     * Modify data with new data
     * 
     * @param array $modification
     * @return $this
     */
    public function modify(array $modification);
    
    /**
     * Copy data from another model
     * 
     * @param \Shavshukov\EventAppBase\Interfaces\ModelInterface $model
     * @return $this
     */
    public function setDataFrom(ModelInterface $model);
    
}
