<?php

namespace Shavshukov\EventAppBase\Interfaces;

interface StorageInterface
{
    /**
     * Get current contents, mostly for higher level storage
     * 
     * @return mixed
     */
    public function content();

    /**
     * Get source of current underlying contents, mostly for backend storage
     * 
     * @return mixed
     */
    public function source();
    
    /**
     * Accept packed higher level storage data into backend lower level storage
     * 
     * @param \Shavshukov\EventAppBase\Interfaces\StorageInterface $storage
     * @reutrn bool
     */
    public function modifyBy(StorageInterface $storage): bool;
    
    /**
     * Set back-end storage for this storage
     * 
     * @param \Shavshukov\EventAppBase\Interfaces\StorageInterface $backend
     */
    public function setBackEnd(StorageInterface $backend);

    /**
     * Load data from the storage
     * 
     * @return bool
     */
    public function load(): bool;
    
    /**
     * Load the data from storage in a non-blocking way
     * 
     * @return \React\Promise\PromiseInterface<bool>
     */
    public function loadAsync(): \React\Promise\PromiseInterface;
    
    /**
     * Save data into the storage, if modified
     * 
     * @return bool
     */
    public function save(): bool;
    
}
