<?php

namespace Shavshukov\EventAppBase\Model;

use Shavshukov\EventAppBase\Interfaces\ValidatableInterface;
use Shavshukov\EventAppBase\Interfaces\ValidationResultInterface;

class ValidationResult implements ValidationResultInterface
{
    /**
     * The object that was validated
     *
     * @var \Shavshukov\EventAppBase\Interfaces\ValidatableInterface
     */
    protected $subject;
    
    /**
     * Invalid fields, which are mandatory and not present
     *
     * @var array|null
     */
    protected $invalidFields;
    
    /**
     * Constructor
     * 
     * @param \Shavshukov\EventAppBase\Interfaces\ValidatableInterface $subject
     * @param array $invalidFields
     */
    public function __construct(ValidatableInterface $subject, array $invalidFields)
    {
        $this->subject = $subject;
        $this->invalidFields = $invalidFields;
    }
    
    /**
     * Check whether all mandatory fields are present
     * 
     * @return bool
     */
    public function isValid(): bool
    {
        return empty($this->invalidFields);
    }
 
    /**
     * Get invalid fields, which are mandatory and not present
     * 
     * @return array
     */
    public function getInvalidFields(): array
    {
        return $this->invalidFields;
    }
    
    /**
     * Get the object that was validated
     * 
     * @return \Shavshukov\EventAppBase\Interfaces\ValidatableInterface
     */
    public function getSubject(): ValidatableInterface
    {
        return $this->subject;
    }
    
}
